@@ -2,13 +2,11 @@
 ** fls
 ** The Sleuth Kit 
 **
-** $Date: 2007/12/20 16:17:46 $
-**
 ** Given an image and directory inode, display the file names and 
 ** directories that exist (both active and deleted)
 **
-** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
+** Brian Carrier [carrier <at> sleuthkit [dot] org]
+** Copyright (c) 2006-2008 Brian Carrier, Basis Technology.  All Rights reserved
 ** Copyright (c) 2003-2005 Brian Carier.  All rights reserved
 **
 ** TASK
@@ -21,11 +19,10 @@
 ** This software is distributed under the Common Public License 1.0
 **
 */
-#include "tsk/tsk_tools_i.h"
+#include "tsk3/tsk_tools_i.h"
 #include <locale.h>
 #include <time.h>
 
-
 static TSK_TCHAR *progname;
 
 void
@@ -66,97 +63,120 @@ usage()
 }
 
 int
-MAIN(int argc, TSK_TCHAR ** argv)
+main(int argc, char ** argv1)
 {
-    TSK_TCHAR *fstype = NULL;
-    TSK_TCHAR *imgtype = NULL;
-    TSK_INUM_T inode;
-    int flags = TSK_FS_DENT_FLAG_ALLOC | TSK_FS_DENT_FLAG_UNALLOC;
-    int ch;
-    TSK_FS_INFO *fs;
-    extern int optind;
+    TSK_IMG_TYPE_ENUM imgtype = TSK_IMG_TYPE_DETECT;
     TSK_IMG_INFO *img;
-    int lclflags;
+
+    TSK_OFF_T imgoff = 0;
+    TSK_FS_TYPE_ENUM fstype = TSK_FS_TYPE_DETECT;
+    TSK_FS_INFO *fs;
+
+    TSK_INUM_T inode;
+    int name_flags = TSK_FS_NAME_FLAG_ALLOC | TSK_FS_NAME_FLAG_UNALLOC;
+    int ch;
+    extern int OPTIND;
+    int fls_flags;
     int32_t sec_skew = 0;
     static TSK_TCHAR *macpre = NULL;
-    TSK_OFF_T imgoff = 0;
-
+    TSK_TCHAR **argv;
+    
+#ifdef TSK_WIN32
+    // On Windows, get the wide arguments (mingw doesn't support wmain)
+    argv = CommandLineToArgvW(GetCommandLineW(), &argc);
+    if( argv == NULL) {    
+        fprintf(stderr, "Error getting wide arguments\n");
+        exit(1);
+    }
+#else
+    argv = (TSK_TCHAR **)argv1;
+#endif
+    
     progname = argv[0];
     setlocale(LC_ALL, "");
 
-    lclflags = TSK_FS_FLS_DIR | TSK_FS_FLS_FILE;
+    fls_flags = TSK_FS_FLS_DIR | TSK_FS_FLS_FILE;
 
-    while ((ch = getopt(argc, argv, _TSK_T("adDf:Fi:m:lo:prs:uvVz:"))) > 0) {
+    while ((ch = GETOPT(argc, argv, _TSK_T("adDf:Fi:m:lo:prs:uvVz:"))) > 0) {
         switch (ch) {
         case _TSK_T('?'):
         default:
             TFPRINTF(stderr, _TSK_T("Invalid argument: %s\n"),
-                argv[optind]);
+                argv[OPTIND]);
             usage();
         case _TSK_T('a'):
-            lclflags |= TSK_FS_FLS_DOT;
+            fls_flags |= TSK_FS_FLS_DOT;
             break;
         case _TSK_T('d'):
-            flags &= ~TSK_FS_DENT_FLAG_ALLOC;
+            name_flags &= ~TSK_FS_NAME_FLAG_ALLOC;
             break;
         case _TSK_T('D'):
-            lclflags &= ~TSK_FS_FLS_FILE;
-            lclflags |= TSK_FS_FLS_DIR;
+            fls_flags &= ~TSK_FS_FLS_FILE;
+            fls_flags |= TSK_FS_FLS_DIR;
             break;
         case _TSK_T('f'):
-            fstype = optarg;
-            if (TSTRCMP(fstype, _TSK_T("list")) == 0) {
-                tsk_fs_print_types(stderr);
+            if (TSTRCMP(OPTARG, _TSK_T("list")) == 0) {
+                tsk_fs_type_print(stderr);
                 exit(1);
             }
-
+            fstype = tsk_fs_type_toid(OPTARG);
+            if (fstype == TSK_FS_TYPE_UNSUPP) {
+                TFPRINTF(stderr,
+                    _TSK_T("Unsupported file system type: %s\n"), OPTARG);
+                usage();
+            }
             break;
         case _TSK_T('F'):
-            lclflags &= ~TSK_FS_FLS_DIR;
-            lclflags |= TSK_FS_FLS_FILE;
+            fls_flags &= ~TSK_FS_FLS_DIR;
+            fls_flags |= TSK_FS_FLS_FILE;
             break;
         case _TSK_T('i'):
-            imgtype = optarg;
-            if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
-                tsk_img_print_types(stderr);
+            if (TSTRCMP(OPTARG, _TSK_T("list")) == 0) {
+                tsk_img_type_print(stderr);
                 exit(1);
             }
+            imgtype = tsk_img_type_toid(OPTARG);
+            if (imgtype == TSK_IMG_TYPE_UNSUPP) {
+                TFPRINTF(stderr, _TSK_T("Unsupported image type: %s\n"),
+                    OPTARG);
+                usage();
+            }
             break;
         case _TSK_T('l'):
-            lclflags |= TSK_FS_FLS_LONG;
+            fls_flags |= TSK_FS_FLS_LONG;
             break;
         case _TSK_T('m'):
-            lclflags |= TSK_FS_FLS_MAC;
-            macpre = optarg;
+            fls_flags |= TSK_FS_FLS_MAC;
+            macpre = OPTARG;
             break;
         case _TSK_T('o'):
-            if ((imgoff = tsk_parse_offset(optarg)) == -1) {
+            if ((imgoff = tsk_parse_offset(OPTARG)) == -1) {
                 tsk_error_print(stderr);
                 exit(1);
             }
             break;
         case _TSK_T('p'):
-            lclflags |= TSK_FS_FLS_FULL;
+            fls_flags |= TSK_FS_FLS_FULL;
             break;
         case _TSK_T('r'):
-            flags |= TSK_FS_DENT_FLAG_RECURSE;
+            name_flags |= TSK_FS_DIR_WALK_FLAG_RECURSE;
             break;
         case _TSK_T('s'):
-            sec_skew = TATOI(optarg);
+            sec_skew = TATOI(OPTARG);
             break;
         case _TSK_T('u'):
-            flags &= ~TSK_FS_DENT_FLAG_UNALLOC;
+            name_flags &= ~TSK_FS_NAME_FLAG_UNALLOC;
             break;
         case _TSK_T('v'):
             tsk_verbose++;
             break;
         case _TSK_T('V'):
-            tsk_print_version(stdout);
+            tsk_version_print(stdout);
             exit(0);
         case 'z':
             {
                 TSK_TCHAR envstr[32];
-                TSNPRINTF(envstr, 32, _TSK_T("TZ=%s"), optarg);
+                TSNPRINTF(envstr, 32, _TSK_T("TZ=%s"), OPTARG);
                 if (0 != PUTENV(envstr)) {
                     tsk_fprintf(stderr, "error setting environment");
                     exit(1);
@@ -171,7 +191,7 @@ MAIN(int argc, TSK_TCHAR ** argv)
     }
 
     /* We need at least one more argument */
-    if (optind == argc) {
+    if (OPTIND == argc) {
         tsk_fprintf(stderr, "Missing image name\n");
         usage();
     }
@@ -180,18 +200,18 @@ MAIN(int argc, TSK_TCHAR ** argv)
     /* Set the full flag to print the full path name if recursion is
      ** set and we are only displaying files or deleted files
      */
-    if ((flags & TSK_FS_DENT_FLAG_RECURSE)
-        && (((flags & TSK_FS_DENT_FLAG_UNALLOC)
-                && (!(flags & TSK_FS_DENT_FLAG_ALLOC)))
-            || ((lclflags & TSK_FS_FLS_FILE)
-                && (!(lclflags & TSK_FS_FLS_DIR))))) {
+    if ((name_flags & TSK_FS_DIR_WALK_FLAG_RECURSE)
+        && (((name_flags & TSK_FS_NAME_FLAG_UNALLOC)
+                && (!(name_flags & TSK_FS_NAME_FLAG_ALLOC)))
+            || ((fls_flags & TSK_FS_FLS_FILE)
+                && (!(fls_flags & TSK_FS_FLS_DIR))))) {
 
-        lclflags |= TSK_FS_FLS_FULL;
+        fls_flags |= TSK_FS_FLS_FULL;
     }
 
     /* set flag to save full path for mactimes style printing */
-    if (lclflags & TSK_FS_FLS_MAC) {
-        lclflags |= TSK_FS_FLS_FULL;
+    if (fls_flags & TSK_FS_FLS_MAC) {
+        fls_flags |= TSK_FS_FLS_FULL;
     }
 
     /* we need to append a / to the end of the directory if
@@ -211,11 +231,12 @@ MAIN(int argc, TSK_TCHAR ** argv)
      *
      * Check the final argument and see if it is a number
      */
-    if (tsk_parse_inum(argv[argc - 1], &inode, NULL, NULL, NULL)) {
+    if (tsk_fs_parse_inum(argv[argc - 1], &inode, NULL, NULL, NULL, NULL)) {
         /* Not an inode at the end */
         if ((img =
-                tsk_img_open(imgtype, argc - optind,
-                    (const TSK_TCHAR **) &argv[optind])) == NULL) {
+                tsk_img_open(argc - OPTIND,
+                    (const TSK_TCHAR **) &argv[OPTIND],
+                    imgtype)) == NULL) {
             tsk_error_print(stderr);
             exit(1);
         }
@@ -225,10 +246,10 @@ MAIN(int argc, TSK_TCHAR ** argv)
                 PRIu64 ")\n", img->size / 512);
             exit(1);
         }
-        if ((fs = tsk_fs_open(img, imgoff, fstype)) == NULL) {
+        if ((fs = tsk_fs_open_img(img, imgoff, fstype)) == NULL) {
             tsk_error_print(stderr);
             if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
-                tsk_fs_print_types(stderr);
+                tsk_fs_type_print(stderr);
 
             img->close(img);
             exit(1);
@@ -237,14 +258,15 @@ MAIN(int argc, TSK_TCHAR ** argv)
     }
     else {
         // check that we have enough arguments
-        if (optind + 1 == argc) {
+        if (OPTIND + 1 == argc) {
             tsk_fprintf(stderr, "Missing image name or inode\n");
             usage();
         }
 
         if ((img =
-                tsk_img_open(imgtype, argc - optind - 1,
-                    (const TSK_TCHAR **) &argv[optind])) == NULL) {
+                tsk_img_open(argc - OPTIND - 1,
+                    (const TSK_TCHAR **) &argv[OPTIND],
+                    imgtype)) == NULL) {
             tsk_error_print(stderr);
             exit(1);
         }
@@ -256,16 +278,17 @@ MAIN(int argc, TSK_TCHAR ** argv)
         }
 
 
-        if ((fs = tsk_fs_open(img, imgoff, fstype)) == NULL) {
+        if ((fs = tsk_fs_open_img(img, imgoff, fstype)) == NULL) {
             tsk_error_print(stderr);
             if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
-                tsk_fs_print_types(stderr);
+                tsk_fs_type_print(stderr);
             img->close(img);
             exit(1);
         }
     }
 
-    if (tsk_fs_fls(fs, lclflags, inode, flags, macpre, sec_skew)) {
+    if (tsk_fs_fls(fs, (TSK_FS_FLS_FLAG_ENUM) fls_flags, inode,
+            (TSK_FS_NAME_FLAG_ENUM) name_flags, macpre, sec_skew)) {
         tsk_error_print(stderr);
         fs->close(fs);
         img->close(img);
