@@ -2,22 +2,18 @@
 ** fsstat
 ** The Sleuth Kit 
 **
-** $Date: 2007/12/20 20:32:37 $
-**
-** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
+** Brian Carrier [carrier <at> sleuthkit [dot] org]
+** Copyright (c) 2006-2008 Brian Carrier, Basis Technology.  All Rights reserved
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
 **
 ** TASK
 ** Copyright (c) 2002 Brian Carrier, @stake Inc. All Rights reserved
 **
-**
 ** This software is distributed under the Common Public License 1.0
-**
 */
-#include "tsk/tsk_tools_i.h"
-#include <locale.h>
 
+#include "tsk3/tsk_tools_i.h"
+#include <locale.h>
 
 static TSK_TCHAR *progname;
 
@@ -43,45 +39,69 @@ usage()
 
 
 int
-MAIN(int argc, TSK_TCHAR ** argv)
+main(int argc, char ** argv1)
 {
-    TSK_FS_INFO *fs;
+    TSK_IMG_TYPE_ENUM imgtype = TSK_IMG_TYPE_DETECT;
     TSK_IMG_INFO *img;
-    TSK_TCHAR *fstype = NULL;
-    TSK_TCHAR *imgtype = NULL;
+
+    TSK_OFF_T imgoff = 0;
+    TSK_FS_TYPE_ENUM fstype = TSK_FS_TYPE_DETECT;
+    TSK_FS_INFO *fs;
+
     int ch;
     uint8_t type = 0;
-    TSK_OFF_T imgoff = 0;
+    TSK_TCHAR **argv;
+    
+#ifdef TSK_WIN32
+    // On Windows, get the wide arguments (mingw doesn't support wmain)
+    argv = CommandLineToArgvW(GetCommandLineW(), &argc);
+    if( argv == NULL) {    
+        fprintf(stderr, "Error getting wide arguments\n");
+        exit(1);
+    }
+#else
+    argv = (TSK_TCHAR **)argv1;
+#endif    
 
     progname = argv[0];
     setlocale(LC_ALL, "");
 
-    while ((ch = getopt(argc, argv, _TSK_T("f:i:o:tvV"))) > 0) {
+    while ((ch = GETOPT(argc, argv, _TSK_T("f:i:o:tvV"))) > 0) {
         switch (ch) {
         case _TSK_T('?'):
         default:
             TFPRINTF(stderr, _TSK_T("Invalid argument: %s\n"),
-                argv[optind]);
+                argv[OPTIND]);
             usage();
 
         case _TSK_T('f'):
-            fstype = optarg;
-            if (TSTRCMP(fstype, _TSK_T("list")) == 0) {
-                tsk_fs_print_types(stderr);
+            if (TSTRCMP(OPTARG, _TSK_T("list")) == 0) {
+                tsk_fs_type_print(stderr);
                 exit(1);
             }
+            fstype = tsk_fs_type_toid(OPTARG);
+            if (fstype == TSK_FS_TYPE_UNSUPP) {
+                TFPRINTF(stderr,
+                    _TSK_T("Unsupported file system type: %s\n"), OPTARG);
+                usage();
+            }
             break;
 
         case _TSK_T('i'):
-            imgtype = optarg;
-            if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
-                tsk_img_print_types(stderr);
+            if (TSTRCMP(OPTARG, _TSK_T("list")) == 0) {
+                tsk_img_type_print(stderr);
                 exit(1);
             }
+            imgtype = tsk_img_type_toid(OPTARG);
+            if (imgtype == TSK_IMG_TYPE_UNSUPP) {
+                TFPRINTF(stderr, _TSK_T("Unsupported image type: %s\n"),
+                    OPTARG);
+                usage();
+            }
             break;
 
         case _TSK_T('o'):
-            if ((imgoff = tsk_parse_offset(optarg)) == -1) {
+            if ((imgoff = tsk_parse_offset(OPTARG)) == -1) {
                 tsk_error_print(stderr);
                 exit(1);
             }
@@ -96,20 +116,20 @@ MAIN(int argc, TSK_TCHAR ** argv)
             break;
 
         case _TSK_T('V'):
-            tsk_print_version(stdout);
+            tsk_version_print(stdout);
             exit(0);
         }
     }
 
     /* We need at least one more argument */
-    if (optind >= argc) {
+    if (OPTIND >= argc) {
         tsk_fprintf(stderr, "Missing image name\n");
         usage();
     }
 
     if ((img =
-            tsk_img_open(imgtype, argc - optind,
-                (const TSK_TCHAR **) &argv[optind])) == NULL) {
+            tsk_img_open(argc - OPTIND,
+                (const TSK_TCHAR **) &argv[OPTIND], imgtype)) == NULL) {
         tsk_error_print(stderr);
         exit(1);
     }
@@ -120,16 +140,16 @@ MAIN(int argc, TSK_TCHAR ** argv)
         exit(1);
     }
 
-    if ((fs = tsk_fs_open(img, imgoff, fstype)) == NULL) {
+    if ((fs = tsk_fs_open_img(img, imgoff, fstype)) == NULL) {
         tsk_error_print(stderr);
         if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
-            tsk_fs_print_types(stderr);
+            tsk_fs_type_print(stderr);
         img->close(img);
         exit(1);
     }
 
     if (type) {
-        tsk_printf("%s\n", tsk_fs_get_type(fs->ftype));
+        tsk_printf("%s\n", tsk_fs_type_toname(fs->ftype));
     }
     else {
         if (fs->fsstat(fs, stdout)) {
