@@ -2,20 +2,16 @@
 ** jcat
 ** The Sleuth Kit 
 **
-** $Date: 2007/12/20 16:17:48 $
-**
-** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
+** Brian Carrier [carrier <at> sleuthkit [dot] org]
+** Copyright (c) 2006-2008 Brian Carrier, Basis Technology.  All Rights reserved
 ** Copyright (c) 2004-2005 Brian Carrier.  All rights reserved
 **
 **
 ** This software is distributed under the Common Public License 1.0
-**
 */
-#include "tsk/tsk_tools_i.h"
+#include "tsk3/tsk_tools_i.h"
 #include <locale.h>
 
-
 #ifdef TSK_WIN32
 #include <fcntl.h>
 #endif
@@ -46,44 +42,68 @@ usage()
 
 
 int
-MAIN(int argc, TSK_TCHAR ** argv)
+main(int argc, char ** argv1)
 {
-    TSK_TCHAR *fstype = NULL;
-    TSK_TCHAR *imgtype = NULL;
-    TSK_FS_INFO *fs;
+    TSK_IMG_TYPE_ENUM imgtype = TSK_IMG_TYPE_DETECT;
     TSK_IMG_INFO *img;
+
+    TSK_OFF_T imgoff = 0;
+    TSK_FS_TYPE_ENUM fstype = TSK_FS_TYPE_DETECT;
+    TSK_FS_INFO *fs;
+
     TSK_INUM_T inum;
     int ch;
     TSK_DADDR_T blk;
     TSK_TCHAR *cp;
-    TSK_OFF_T imgoff = 0;
+    TSK_TCHAR **argv;
+    
+#ifdef TSK_WIN32
+    // On Windows, get the wide arguments (mingw doesn't support wmain)
+    argv = CommandLineToArgvW(GetCommandLineW(), &argc);
+    if( argv == NULL) {    
+        fprintf(stderr, "Error getting wide arguments\n");
+        exit(1);
+    }
+#else
+    argv = (TSK_TCHAR **)argv1;
+#endif    
 
     progname = argv[0];
     setlocale(LC_ALL, "");
 
-    while ((ch = getopt(argc, argv, _TSK_T("f:i:o:vV"))) > 0) {
+    while ((ch = GETOPT(argc, argv, _TSK_T("f:i:o:vV"))) > 0) {
         switch (ch) {
         case _TSK_T('?'):
         default:
             TFPRINTF(stderr, _TSK_T("Invalid argument: %s\n"),
-                argv[optind]);
+                argv[OPTIND]);
             usage();
         case _TSK_T('f'):
-            fstype = optarg;
-            if (TSTRCMP(fstype, _TSK_T("list")) == 0) {
-                tsk_fs_print_types(stderr);
+            if (TSTRCMP(OPTARG, _TSK_T("list")) == 0) {
+                tsk_fs_type_print(stderr);
                 exit(1);
             }
+            fstype = tsk_fs_type_toid(OPTARG);
+            if (fstype == TSK_FS_TYPE_UNSUPP) {
+                TFPRINTF(stderr,
+                    _TSK_T("Unsupported file system type: %s\n"), OPTARG);
+                usage();
+            }
             break;
         case _TSK_T('i'):
-            imgtype = optarg;
-            if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
-                tsk_img_print_types(stderr);
+            if (TSTRCMP(OPTARG, _TSK_T("list")) == 0) {
+                tsk_img_type_print(stderr);
                 exit(1);
             }
+            imgtype = tsk_img_type_toid(OPTARG);
+            if (imgtype == TSK_IMG_TYPE_UNSUPP) {
+                TFPRINTF(stderr, _TSK_T("Unsupported image type: %s\n"),
+                    OPTARG);
+                usage();
+            }
             break;
         case _TSK_T('o'):
-            if ((imgoff = tsk_parse_offset(optarg)) == -1) {
+            if ((imgoff = tsk_parse_offset(OPTARG)) == -1) {
                 tsk_error_print(stderr);
                 exit(1);
             }
@@ -92,13 +112,13 @@ MAIN(int argc, TSK_TCHAR ** argv)
             tsk_verbose++;
             break;
         case _TSK_T('V'):
-            tsk_print_version(stdout);
+            tsk_version_print(stdout);
             exit(0);
         }
     }
 
     /* We need at least two more arguments */
-    if (optind + 1 >= argc) {
+    if (OPTIND + 1 >= argc) {
         tsk_fprintf(stderr, "Missing image name and/or block address\n");
         usage();
     }
@@ -110,11 +130,12 @@ MAIN(int argc, TSK_TCHAR ** argv)
     }
 
     /* Do we have an inode as well? */
-    if (tsk_parse_inum(argv[argc - 2], &inum, NULL, NULL, NULL)) {
+    if (tsk_fs_parse_inum(argv[argc - 2], &inum, NULL, NULL, NULL, NULL)) {
         /* Not a number therefore an image */
         if ((img =
-                tsk_img_open(imgtype, argc - optind - 1,
-                    (const TSK_TCHAR **) &argv[optind])) == NULL) {
+                tsk_img_open(argc - OPTIND - 1,
+                    (const TSK_TCHAR **) &argv[OPTIND],
+                    imgtype)) == NULL) {
             tsk_error_print(stderr);
             exit(1);
         }
@@ -125,10 +146,10 @@ MAIN(int argc, TSK_TCHAR ** argv)
             exit(1);
         }
 
-        if ((fs = tsk_fs_open(img, imgoff, fstype)) == NULL) {
+        if ((fs = tsk_fs_open_img(img, imgoff, fstype)) == NULL) {
             tsk_error_print(stderr);
             if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
-                tsk_fs_print_types(stderr);
+                tsk_fs_type_print(stderr);
             img->close(img);
             exit(1);
         }
@@ -136,16 +157,17 @@ MAIN(int argc, TSK_TCHAR ** argv)
     }
     else {
         if ((img =
-                tsk_img_open(imgtype, argc - optind - 2,
-                    (const TSK_TCHAR **) &argv[optind])) == NULL) {
+                tsk_img_open(argc - OPTIND - 2,
+                    (const TSK_TCHAR **) &argv[OPTIND],
+                    imgtype)) == NULL) {
             tsk_error_print(stderr);
             exit(1);
         }
 
-        if ((fs = tsk_fs_open(img, imgoff, fstype)) == NULL) {
+        if ((fs = tsk_fs_open_img(img, imgoff, fstype)) == NULL) {
             tsk_error_print(stderr);
             if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
-                tsk_fs_print_types(stderr);
+                tsk_fs_type_print(stderr);
             img->close(img);
             exit(1);
         }
@@ -174,15 +196,16 @@ MAIN(int argc, TSK_TCHAR ** argv)
             "Journal support does not exist for this file system\n");
         fs->close(fs);
         img->close(img);
-        return 1;
+        exit(1);
     }
 
 #ifdef TSK_WIN32
     if (-1 == _setmode(_fileno(stdout), _O_BINARY)) {
-        tsk_errno = TSK_ERR_FS_WRITE;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
+        fprintf(stderr,
             "jcat: error setting stdout to binary: %s", strerror(errno));
-        return 1;
+        fs->close(fs);
+        img->close(img);
+        exit(1);
     }
 #endif
 
