@@ -1,20 +1,15 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2007/12/20 16:17:49 $
- *
- * Brian Carrier [carrier@sleuthkit.org]
- * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
+ * Brian Carrier [carrier <at> sleuthkit [dot] org]
+ * Copyright (c) 2006-2008 Brian Carrier, Basis Technology.  All rights reserved
  * Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
  *
  * mmls - list media management structure contents
  *
- *
  * This software is distributed under the Common Public License 1.0
- *
  */
-#include "tsk/tsk_tools_i.h"
-
+#include "tsk3/tsk_tools_i.h"
 
 static TSK_TCHAR *progname;
 
@@ -29,10 +24,10 @@ usage()
 {
     TFPRINTF(stderr,
         _TSK_T
-        ("%s [-i imgtype] [-o imgoffset] [-brvV] [-t mmtype] image [images]\n"),
+        ("%s [-i imgtype] [-o imgoffset] [-brvV] [-aAmM] [-t vstype] image [images]\n"),
         progname);
     tsk_fprintf(stderr,
-        "\t-t mmtype: The type of partition system (use '-t list' for list of supported types)\n");
+        "\t-t vstype: The type of volume system (use '-t list' for list of supported types)\n");
     tsk_fprintf(stderr,
         "\t-i imgtype: The format of the image file (use '-i list' for list supported types)\n");
     tsk_fprintf(stderr,
@@ -42,6 +37,12 @@ usage()
         "\t-r: recurse and look for other partition tables in partitions (DOS Only)\n");
     tsk_fprintf(stderr, "\t-v: verbose output\n");
     tsk_fprintf(stderr, "\t-V: print the version\n");
+    tsk_fprintf(stderr,
+        "Unless any of these are specified, all volume types are shown\n");
+    tsk_fprintf(stderr, "\t-a: Show allocated volumes\n");
+    tsk_fprintf(stderr, "\t-A: Show unallocated volumes\n");
+    tsk_fprintf(stderr, "\t-m: Show metadata volumes\n");
+    tsk_fprintf(stderr, "\t-M: Hide metadata volumes\n");
     exit(1);
 }
 
@@ -51,26 +52,28 @@ usage()
  * Prints the layout information
  * */
 static TSK_WALK_RET_ENUM
-part_act(TSK_MM_INFO * mm, TSK_PNUM_T pnum, TSK_MM_PART * part, int flag,
-    void *ptr)
+part_act(TSK_VS_INFO * vs, const TSK_VS_PART_INFO * part, void *ptr)
 {
+    if (part->flags & TSK_VS_PART_FLAG_META)
+        tsk_printf("%.2" PRIuPNUM ":  Meta    ", part->addr);
+
     /* Neither table or slot were given */
-    if ((part->table_num == -1) && (part->slot_num == -1))
-        tsk_printf("%.2" PRIuPNUM ":  -----   ", pnum);
+    else if ((part->table_num == -1) && (part->slot_num == -1))
+        tsk_printf("%.2" PRIuPNUM ":  -----   ", part->addr);
 
     /* Table was not given, but slot was */
     else if ((part->table_num == -1) && (part->slot_num != -1))
         tsk_printf("%.2" PRIuPNUM ":  %.2" PRIu8 "      ",
-            pnum, part->slot_num);
+            part->addr, part->slot_num);
 
     /* The Table was given, but slot wasn't */
     else if ((part->table_num != -1) && (part->slot_num == -1))
-        tsk_printf("%.2" PRIuPNUM ":  -----   ", pnum);
+        tsk_printf("%.2" PRIuPNUM ":  -----   ", part->addr);
 
     /* Both table and slot were given */
     else if ((part->table_num != -1) && (part->slot_num != -1))
         tsk_printf("%.2" PRIuPNUM ":  %.2d:%.2d   ",
-            pnum, part->table_num, part->slot_num);
+            part->addr, part->table_num, part->slot_num);
 
     if (print_bytes) {
         TSK_OFF_T size;
@@ -101,11 +104,10 @@ part_act(TSK_MM_INFO * mm, TSK_PNUM_T pnum, TSK_MM_PART * part, int flag,
         /* Print the layout */
         tsk_printf("%.10" PRIuDADDR "   %.10" PRIuDADDR "   %.10" PRIuDADDR
             "   %.4" PRIuOFF "%c   %s\n", part->start,
-            (TSK_DADDR_T) (part->start + part->len - 1), part->len, size, unit,
-            part->desc);
+            (TSK_DADDR_T) (part->start + part->len - 1), part->len, size,
+            unit, part->desc);
     }
     else {
-
         /* Print the layout */
         tsk_printf("%.10" PRIuDADDR "   %.10" PRIuDADDR "   %.10" PRIuDADDR
             "   %s\n", part->start,
@@ -113,8 +115,8 @@ part_act(TSK_MM_INFO * mm, TSK_PNUM_T pnum, TSK_MM_PART * part, int flag,
             part->desc);
     }
 
-    if ((recurse) && (mm->mmtype == TSK_MM_INFO_TYPE_DOS)
-        && (part->type == TSK_MM_PART_TYPE_VOL)) {
+    if ((recurse) && (vs->vstype == TSK_VS_TYPE_DOS)
+        && (part->flags == TSK_VS_PART_FLAG_ALLOC)) {
         // @@@ This assumes 512-byte sectors
         if (recurse_cnt < 64)
             recurse_list[recurse_cnt++] = part->start * 512;
@@ -124,12 +126,12 @@ part_act(TSK_MM_INFO * mm, TSK_PNUM_T pnum, TSK_MM_PART * part, int flag,
 }
 
 static void
-print_header(TSK_MM_INFO * mm)
+print_header(const TSK_VS_INFO * vs)
 {
-    tsk_printf("%s\n", mm->str_type);
+    tsk_printf("%s\n", tsk_vs_type_todesc(vs->vstype));
     tsk_printf("Offset Sector: %" PRIuDADDR "\n",
-        (TSK_DADDR_T) (mm->offset / mm->block_size));
-    tsk_printf("Units are in %d-byte sectors\n\n", mm->block_size);
+        (TSK_DADDR_T) (vs->offset / vs->block_size));
+    tsk_printf("Units are in %d-byte sectors\n\n", vs->block_size);
     if (print_bytes)
         tsk_printf
             ("     Slot    Start        End          Length       Size    Description\n");
@@ -140,34 +142,64 @@ print_header(TSK_MM_INFO * mm)
 
 
 int
-MAIN(int argc, TSK_TCHAR ** argv)
+main(int argc, char ** argv1)
 {
-    TSK_MM_INFO *mm;
-    TSK_TCHAR *mmtype = NULL;
+    TSK_VS_INFO *vs;
     int ch;
     TSK_OFF_T imgoff = 0;
-    uint8_t flags = 0;
-    TSK_TCHAR *imgtype = NULL;
+    int flags = 0;
     TSK_IMG_INFO *img;
+    TSK_IMG_TYPE_ENUM imgtype = TSK_IMG_TYPE_DETECT;
+    TSK_VS_TYPE_ENUM vstype = TSK_VS_TYPE_DETECT;
+    uint8_t hide_meta = 0;
+    TSK_TCHAR **argv;
+    
+#ifdef TSK_WIN32
+    // On Windows, get the wide arguments (mingw doesn't support wmain)
+    argv = CommandLineToArgvW(GetCommandLineW(), &argc);
+    if( argv == NULL) {    
+        fprintf(stderr, "Error getting wide arguments\n");
+        exit(1);
+    }
+#else
+    argv = (TSK_TCHAR **)argv1;
+#endif
+    
 
     progname = argv[0];
 
-    while ((ch = getopt(argc, argv, _TSK_T("bi:o:rt:vV"))) > 0) {
+    while ((ch = GETOPT(argc, argv, _TSK_T("aAbi:mMo:rt:vV"))) > 0) {
         switch (ch) {
+        case _TSK_T('a'):
+            flags |= TSK_VS_PART_FLAG_ALLOC;
+            break;
+        case _TSK_T('A'):
+            flags |= TSK_VS_PART_FLAG_UNALLOC;
+            break;
         case _TSK_T('b'):
             print_bytes = 1;
             break;
         case _TSK_T('i'):
-            imgtype = optarg;
-            if (TSTRCMP(imgtype, _TSK_T("list")) == 0) {
-                tsk_img_print_types(stderr);
+            if (TSTRCMP(OPTARG, _TSK_T("list")) == 0) {
+                tsk_img_type_print(stderr);
                 exit(1);
             }
-
+            imgtype = tsk_img_type_toid(OPTARG);
+            if (imgtype == TSK_IMG_TYPE_UNSUPP) {
+                TFPRINTF(stderr, _TSK_T("Unsupported image type: %s\n"),
+                    OPTARG);
+                usage();
+            }
+            break;
+        case _TSK_T('m'):
+            flags |= (TSK_VS_PART_FLAG_META);
+            break;
+        case _TSK_T('M'):
+            // we'll set this after all flags have been set
+            hide_meta = 1;
             break;
-
         case _TSK_T('o'):
-            if ((imgoff = tsk_parse_offset(optarg)) == -1) {
+            if ((imgoff = tsk_parse_offset(OPTARG)) == -1) {
                 tsk_error_print(stderr);
                 exit(1);
             }
@@ -176,18 +208,23 @@ MAIN(int argc, TSK_TCHAR ** argv)
             recurse = 1;
             break;
         case _TSK_T('t'):
-            mmtype = optarg;
-            if (TSTRCMP(mmtype, _TSK_T("list")) == 0) {
-                tsk_mm_print_types(stderr);
+            if (TSTRCMP(OPTARG, _TSK_T("list")) == 0) {
+                tsk_vs_type_print(stderr);
                 exit(1);
             }
-
+            vstype = tsk_vs_type_toid(OPTARG);
+            if (vstype == TSK_VS_TYPE_UNSUPP) {
+                TFPRINTF(stderr,
+                    _TSK_T("Unsupported volume system type: %s\n"),
+                    OPTARG);
+                usage();
+            }
             break;
         case _TSK_T('v'):
             tsk_verbose++;
             break;
         case _TSK_T('V'):
-            tsk_print_version(stdout);
+            tsk_version_print(stdout);
             exit(0);
         case _TSK_T('?'):
         default:
@@ -196,16 +233,27 @@ MAIN(int argc, TSK_TCHAR ** argv)
         }
     }
 
+    // if they want to hide metadata volumes, set that now
+    if (hide_meta) {
+        if (flags == 0)
+            flags = (TSK_VS_PART_FLAG_ALLOC | TSK_VS_PART_FLAG_UNALLOC);
+        else
+            flags &= ~TSK_VS_PART_FLAG_META;
+    }
+    else if (flags == 0) {
+        flags = TSK_VS_PART_FLAG_ALL;
+    }
+
     /* We need at least one more argument */
-    if (optind >= argc) {
+    if (OPTIND >= argc) {
         tsk_fprintf(stderr, "Missing image name\n");
         usage();
     }
 
     /* open the image */
     img =
-        tsk_img_open(imgtype, argc - optind,
-        (const TSK_TCHAR **) &argv[optind]);
+        tsk_img_open(argc - OPTIND,
+        (const TSK_TCHAR **) &argv[OPTIND], imgtype);
 
     if (img == NULL) {
         tsk_error_print(stderr);
@@ -219,40 +267,42 @@ MAIN(int argc, TSK_TCHAR ** argv)
     }
 
     /* process the partition tables */
-    mm = tsk_mm_open(img, (TSK_OFF_T) imgoff, mmtype);
-    if (mm == NULL) {
+    vs = tsk_vs_open(img, (TSK_OFF_T) imgoff, vstype);
+    if (vs == NULL) {
         tsk_error_print(stderr);
-        if (tsk_errno == TSK_ERR_MM_UNSUPTYPE)
-            tsk_mm_print_types(stderr);
+        if (tsk_errno == TSK_ERR_VS_UNSUPTYPE)
+            tsk_vs_type_print(stderr);
+        tsk_img_close(img);
         exit(1);
     }
 
-    print_header(mm);
+    print_header(vs);
 
-    if (mm->part_walk(mm, mm->first_part, mm->last_part, flags,
-            part_act, NULL)) {
+    if (tsk_vs_part_walk(vs, 0, vs->part_count - 1,
+            (TSK_VS_PART_FLAG_ENUM) flags, part_act, NULL)) {
         tsk_error_print(stderr);
-        mm->close(mm);
+        tsk_vs_close(vs);
+        tsk_img_close(img);
         exit(1);
     }
 
-    mm->close(mm);
-    if ((recurse) && (mm->mmtype == TSK_MM_INFO_TYPE_DOS)) {
+    tsk_vs_close(vs);
+    if ((recurse) && (vs->vstype == TSK_VS_TYPE_DOS)) {
         int i;
         /* disable recursing incase we hit another DOS partition
          * future versions may support more layers */
         recurse = 0;
 
         for (i = 0; i < recurse_cnt; i++) {
-            mm = tsk_mm_open(img, recurse_list[i], NULL);
-            if (mm != NULL) {
+            vs = tsk_vs_open(img, recurse_list[i], TSK_VS_TYPE_DETECT);
+            if (vs != NULL) {
                 tsk_printf("\n\n");
-                print_header(mm);
-                if (mm->part_walk(mm, mm->first_part, mm->last_part, flags,
-                        part_act, NULL)) {
+                print_header(vs);
+                if (tsk_vs_part_walk(vs, 0, vs->part_count - 1,
+                        (TSK_VS_PART_FLAG_ENUM) flags, part_act, NULL)) {
                     tsk_error_reset();
                 }
-                mm->close(mm);
+                tsk_vs_close(vs);
             }
             else {
                 /* Ignore error in this case and reset */
@@ -261,6 +311,6 @@ MAIN(int argc, TSK_TCHAR ** argv)
         }
     }
 
-    img->close(img);
+    tsk_img_close(img);
     exit(0);
 }
