@@ -203,6 +203,8 @@ tsk_fs_dir_open_meta(TSK_FS_INFO * a_fs, TSK_INUM_T a_addr)
     }
 
     retval = a_fs->dir_open_meta(a_fs, &fs_dir, a_addr);
+    if (retval != TSK_OK)
+        return NULL;
 
     return fs_dir;
 }
@@ -776,10 +778,7 @@ find_orphan_meta_walk_cb(TSK_FS_FILE * a_fs_file, void *a_ptr)
     }
     data->fs_name->meta_addr = a_fs_file->meta->addr;
     data->fs_name->flags = TSK_FS_NAME_FLAG_UNALLOC;
-    if (a_fs_file->meta->type == TSK_FS_META_TYPE_DIR)
-        data->fs_name->type = TSK_FS_NAME_TYPE_DIR;
-    else
-        data->fs_name->type = TSK_FS_NAME_TYPE_REG;
+    data->fs_name->type = TSK_FS_NAME_TYPE_UNDEF;
 
     if (tsk_fs_dir_add(data->fs_dir, data->fs_name))
         return TSK_WALK_ERROR;
